//                                         This file is really -*-C++/l-*-
//        Directory:  Homework/Assignment1/Solutions/Code/FSM/
//             File:  MapWithDefault.h
//       Created by:  David Casperson <casper@David-Caspersons-Computer.local>
//          Created:  Sat Oct  6 18:11:03 2007
//
//      Description:  An improvement (from our pov) on std::map
//
//                    This map class is like the standard map class, except
//                    that the []-operator returns a specified default value
//                    if the index has not yet been entered in the map.
//
//                    This makes the map more efficient for data that are
//                    sparse with respect to not being default, and allows
//                    us to supply a const member function operator [].

#if !defined(MapWithDefault_INCLUDED)


#include <string>
#include <map>

template <typename xxx_t, typename value_t, value_t defaultValue=value_t()>
class MapWithDefault
    : public std::map<xxx_t, value_t>
{
    typedef std::map<xxx_t, value_t> base_t ;
public:
    value_t
    operator[] (typename base_t::key_type const& key) const
          {
              typename base_t::const_iterator where(find(key)) ;
              return (where!=base_t::end()) ? where->second : defaultValue ;
          }
    value_t&
    operator[] (typename base_t::key_type const& key) 
          {
              return base_t::operator[](key) ;
          }
} ;

#define MapWithDefault_INCLUDED
#endif// !defined(MapWithDefault_INCLUDED)
